/***************************************************************************
 *   Copyright (C) 2005-2007 by Christian Weilbach                         *
 *   christian_weilbach@web.de                                             *
 *   Copyright (C) 2007 by Antonio Aloisio                                 *
 *   gnuton@gnuton.org                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qcheckbox.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qfileinfo.h>
#include <qvalidator.h>

#include <klineedit.h>
#include <klocale.h>
#include <ktextedit.h>
#include <kmessagebox.h>
#include <kcombobox.h>
#include <kpushbutton.h>
#include <kdatetimewidget.h>
#include <qpixmap.h>
#include <qstylesheet.h>
#include <kdebug.h>
#include <kurl.h>
#include <ktoolbar.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <ktextbrowser.h>
#include <kstatusbar.h>
#include <ksqueezedtextlabel.h>
#include <khtml_part.h>
#include <kiconloader.h>
#include <kpassivepopup.h>
#include <kinputdialog.h>

#include "kbloggerconfigdialog.h"
#include "kbloggerconfig.h"
#include "kbloggerlinkdialog.h"

#include "kblogger.h"
#include "kbloggereditor.h"
#include "API_Blog.h"



KBloggerEditor::KBloggerEditor ( KBlog::APIBlog* backend, QDir* tempDir, int postid, QWidget* parent, const char* name, WFlags fl )
		: KBloggerEditorBlank ( parent,name,fl ), m_parent((KBlogger*)parent), m_postid ( postid ), m_linkDialog(), cssUrl(new KURL(QString::null)), m_tempDir(tempDir)
{
	kdDebug() << "KBloggerEditor::KBloggerEditor()" << endl;
	/*if ( !m_backend ){
		
		this->close();
	}*/

	//Update KTimeStamp Widget
	TimeStampKDateTime->setDateTime ( QDateTime::currentDateTime() );

	m_backend=backend;
	m_linkDialog=new KBloggerLinkDialog ( this );

	kdDebug() << "initialize KActions & Toolbar" << endl;
	m_actionCollection= new KActionCollection ( this );
	m_toolBar= new KToolBar ( VisualPageWidget );
	TextVLayout->addWidget ( m_toolBar, 0, 0 );

	//Add Bold Action
	KAction *boldAction = new KAction ( i18n ( "Bold" ), "bold",
	                                    0,
	                                    this, SLOT ( toggleBold() ),
	                                    m_actionCollection, "bold" );
	boldAction->setIcon ( "text_bold" );
	boldAction->plug ( m_toolBar );

	//Add Italic Action
	KAction *italicAction = new KAction ( i18n ( "Italic" ), "italic",
	                                      0,
	                                      this, SLOT ( toggleItalic() ),
	                                      m_actionCollection, "italic" );
	italicAction->setIcon ( "text_italic" );
	italicAction->plug ( m_toolBar );

	//Add Underline Action
	KAction *underlineAction = new KAction ( i18n ( "Underline" ), "underline",
	                           0,
	                           this, SLOT ( toggleUnderline() ),
	                           m_actionCollection, "underline" );
	underlineAction->setIcon ( "text_under" );
	underlineAction->plug ( m_toolBar );


	//Add StrikeOut Tag Action
	KAction *strikeOutAction = new KAction ( i18n ( "Strike Out" ), "strikeOut",
	                            0,
	                            this, SLOT ( toggleStrikeOut() ),
	                            m_actionCollection, "strikeOut" );
	strikeOutAction->setIcon ( "text_strike" ); //FIXME Choose a better icon
	strikeOutAction->plug ( m_toolBar );

	//Add HTML TAG <Code> Action
	KAction *codeAction = new KAction ( i18n ( "Code" ), "code",
	                                    0,
	                                    this, SLOT ( toggleCode() ),
	                                    m_actionCollection, "code" );
	codeAction->setIcon ( "frame_text" ); //FIXME Choose a better icon
	codeAction->plug ( m_toolBar );

	//Add Link Action
	KAction *addLinkAction = new KAction ( i18n ( "Add Link..." ), "addLink",
	                                       0,
	                                       this, SLOT ( linkDialogSlot() ),
	                                       m_actionCollection, "addLink" );
	addLinkAction->setIcon ( "rss_tag" );//FIXME Choose a better icon
	addLinkAction->plug ( m_toolBar );

	//Add Image Action
	KAction *addImageAction = new KAction ( i18n ( "Add Image..." ), "addLink",
	                                        0,
	                                        this, SLOT ( addImage() ),
	                                        m_actionCollection, "addLink" );
	addImageAction->setIcon ( "frame_image" );//FIXME Choose a better icon
	addImageAction->plug ( m_toolBar );

	//Add Html Tag Action
	KAction *addHtmlTagAction = new KAction ( i18n ( "Add HTML tag..." ), "addHtmlTag",
	                            0,
	                            this, SLOT ( htmlDialogSlot() ),
	                            m_actionCollection, "addHtmlTag" );
	addHtmlTagAction->setIcon ( "wp" ); //FIXME Choose a better icon
	addHtmlTagAction->plug ( m_toolBar );

	//Visual Editor Spell Checker Action
	KAction *checkSpellingAction = new KAction ( i18n ( "Check Spelling" ), "checkSpelling",
	                            0,
	                            this, SLOT ( spellchecking() ),
	                            m_actionCollection, "checkSpelling" );
	checkSpellingAction->setIcon ( "spellcheck" ); 
	checkSpellingAction->plug ( m_toolBar );

	//STATUS BAR!!!! ;)
	m_statusBar= new KStatusBar(this,"status bar");
	//KBloggerEditorBlankLayout->addWidget(m_statusBar,KBloggerEditorBlankLayout->numRows(),0);

	//Preview KHTML Widget
	PreviewBrowser = new KHTMLPart(PreviewPageWidget);
	PreviewPageWidgetLayout->addWidget( (QWidget*)PreviewBrowser->view(), 0, 0 );

	PreviewBrowser->setJScriptEnabled(false);
	PreviewBrowser->setJavaEnabled(false);
	PreviewBrowser->setMetaRefreshEnabled(false);
	PreviewBrowser->setPluginsEnabled(false);
	PreviewBrowser->setProgressInfoEnabled(false);

// TODO either fetch asynchronous or remove it
// TODO make it compatible with @import and other css import possibilities
	//Get Css
// 	KURL url = KBloggerConfig::url();
// 	url.setFileName(QString::null); // now url="www.yourblog.org:80/path/to/"
// 	if (url.isValid()){
// 		
// 		PreviewBrowser->openURL(url);
// 		QRegExp cssRegExp("href=.(http://\\w+\\.css)");
// 		if ( cssRegExp.search( PreviewBrowser->documentSource() ) != -1)
// 			cssUrl=new KURL(cssRegExp.cap());
// 		kdDebug() << "KBloggerEditor::KBloggerEditor url.isValid()==TRUE cssURL=" << cssUrl->url() << endl;
// 	}
// 	
// 	if ( !cssUrl->isValid() ){
// 		QPixmap icon( m_parent->m_iconLoader->loadIcon("kblogger/warning",KIcon::Panel ));
// 		KPassivePopup::message ( "KBlogger Warning","Stylesheet not found!",icon,this,"Passive Error Message" );
// 	}

	
	layoutSend->addWidget(m_statusBar,0,0);
	connect(m_actionCollection, SIGNAL( actionStatusText( const QString & ) ),
                m_statusBar, SLOT( message( const QString & ) ) );
	connect(m_actionCollection, SIGNAL( clearStatusText() ),
                m_statusBar, SLOT( clear() ) );
	connect(m_backend, SIGNAL( error( const QString& ) ),
		m_statusBar, SLOT( message( const QString & ) ) );

	kdDebug() << "interfaceName() " << backend->interfaceName() << endl;

	//FIXME if Spelling check is enabled => several formatting errors in WYSIWYG.
	VisualTextEditor->setCheckSpellingEnabled ( false );

	if ( m_backend->interfaceName() != "Blogger API 1.0" )
	{ // Only Blogger API doesn't support a title and categories
		TitleLineEdit->setReadOnly ( false );
		CategoryComboBox->setEditable ( true );
		TimeStampCheckBox->setEnabled ( true );
		nCategFetched=0;
		KIO::TransferJob *catjob = m_backend->createListCategoriesJob ( KURL::KURL ( KBloggerConfig::blogId() ) );
		m_statusBar->message(i18n("Fetching List of Categories..."));
		connect ( catjob, SIGNAL ( result ( KIO::Job* ) ),
		          m_backend, SLOT ( interpretListCategoriesJob ( KIO::Job* ) ) );
		connect ( m_backend, SIGNAL ( categoryInfoRetrieved ( const QString&, const QString& ) ),
		          this, SLOT ( getCategoriesSlot ( const QString&, const QString& ) ) );
		connect ( m_backend, SIGNAL ( fetchingCategoriesFinished() ),
		          this, SLOT ( disconnectCategoriesSlot() ) );
	}

	kdDebug() << "m_postid: " << m_postid << endl;
	if ( m_postid!=-1 )
	{
		kdDebug() << "Creating new DownloadJob at " << m_postid << endl;
		KIO::TransferJob *editjob = m_backend->createDownloadJob ( KURL::KURL ( QString ( "%1" ).arg ( m_postid ) ) );
		connect ( editjob, SIGNAL ( result ( KIO::Job* ) ),
		          m_backend, SLOT ( interpretDownloadItemsJob ( KIO::Job* ) ) );
		connect ( m_backend, SIGNAL ( itemOnServer ( KBlog::BlogPosting& ) ),
		          this, SLOT ( loadPostSlot ( KBlog::BlogPosting& ) ) );
	}

	connect ( m_linkDialog, SIGNAL ( addLink ( const QString&, const QString& ) ),
	          this, SLOT ( addHtmlLink ( const QString&, const QString& ) ) );

	connect ( m_backend, SIGNAL ( error ( const QString& ) ),
	          this, SLOT ( handleBackendError ( const QString& ) ) );// actually this can be called from elsewhere, too, but this shouldn't be a problem since we only want to release the send button

	show();
}

void KBloggerEditor::handleSuccess ( const int id )
{
	KMessageBox::information ( this, i18n ( "Element %1 posted." ).arg ( id ) );
	emit editorClosed();
	close(true);
}

void KBloggerEditor::handleBackendError ( const QString& error )
{
	SendButton->setEnabled ( false );
}

KBloggerEditor::~KBloggerEditor()
{
	kdDebug() << "KBloggerEditor::~KBloggerEditor()" << endl;
	//delete m_linkDialog; // it's a child of this class then will be auto deleted!
}

void KBloggerEditor::toggleItalic()
{
	VisualTextEditor->setItalic ( !VisualTextEditor->italic() );
}

void KBloggerEditor::toggleBold()
{
	VisualTextEditor->setBold ( !VisualTextEditor->bold() );
}

void KBloggerEditor::toggleUnderline()
{
	VisualTextEditor->setUnderline ( !VisualTextEditor->underline() );
}

void KBloggerEditor::toggleStrikeOut()
{
	QFont fontInVisual(VisualTextEditor->currentFont ());
	fontInVisual.setStrikeOut (!fontInVisual.strikeOut());
	VisualTextEditor->setCurrentFont(fontInVisual);
}

void KBloggerEditor::toggleCode()
{
	static QString preFontFamily;
	if ( VisualTextEditor->family() != "Monospace" ){
		preFontFamily=VisualTextEditor->family();
		VisualTextEditor->setFamily("Monospace");
	}else{
		
		VisualTextEditor->setFamily(preFontFamily);
	}
}

void KBloggerEditor::addImage()
{
	//KMessageBox::sorry ( this, i18n ( "Not implemented yet!" ) );
	
	m_parent->uploadImageSlot();
	connect ( m_parent, SIGNAL(fileUploaded(const QString&)),SLOT(addImageUploaded(const QString&)));
}

void KBloggerEditor::addImageUploaded(const QString& url)
{
	kdDebug() << "KBloggerEditor::addImageUploaded" << endl;
	disconnect ( m_parent, SIGNAL(fileUploaded(const QString&)),this, SLOT(addImageUploaded(const QString&)));
	m_statusBar->message(i18n("Image uploaded with success."));
	if ( KMessageBox::questionYesNo ( this, i18n ( "Do you want use a thumbnail of uploaded image?" ) ) !=3 ){
		insertTag(url,0,0,img); //NOTE QTextEdit accept only <img .... > instead of <img ..../>
	}else{
		QFileInfo uploadedImage(url);
		QString thumbnailImage;
		//Note works only with wordpress?
		thumbnailImage=uploadedImage.dirPath()+"/"+uploadedImage.baseName(true)+".thumbnail."+uploadedImage.extension();
		insertTag(url,uploadedImage.fileName(),thumbnailImage,imglinked);
	}
}

void KBloggerEditor::disconnectCategoriesSlot()
{
	disconnect ( dynamic_cast<QObject*> ( m_backend ), SIGNAL ( categoryInfoRetrieved ( const QString&, const QString& ) ),
	             dynamic_cast<QObject*> ( this ), SLOT ( getCategoriesSlot ( const QString&, const QString& ) ) );
}

void KBloggerEditor::linkDialogSlot()
{
	//TODO: EDIT LINK
	QString selectedText;
	selectedText=VisualTextEditor->selectedText();
	kdDebug() << "KBloggerEditor::linkDialogSlot()" << selectedText << endl;
	if ( selectedText.isEmpty() ) //Add a link at the cursor position
	{
		m_linkDialog->setLinkName (QString::null);
		m_linkDialog->show();
		return;
	}
	else
	{
		QRegExp rxStart("<a\\s?href=\"[^<>]*\\s?>"), rxEnd("</a>");
		if (rxStart.search(selectedText) != -1 || rxEnd.search(selectedText) != -1)
		{ //Link Exists! => UnLink...
		  kdDebug() << "KBloggerEditor::linkDialogSlot()\n" << selectedText << "\n" <<  endl;
		  selectedText.remove(rxStart);
		  selectedText.remove(rxEnd);
		  insertTag(selectedText,0,0,notags);
		}else{ //Add a Link
		  m_linkDialog->setLinkName ( selectedText );
		  m_linkDialog->show();
		}
	};
}

void KBloggerEditor::htmlDialogSlot()
{
	//m_htmlDialog->show();
	QString code;
	QRegExp testRegExp("(<[^<]*>)([^<]*)(</[^<]*>)");
	QValidator* validator = new QRegExpValidator(testRegExp,this);
	code=KInputDialog::getText(i18n("KBlogger - Visual Editor"),i18n("Insert here your HTML code:"),i18n("<tag>text</tag>"),0,this,"AddHtmlDialog",validator);
	insertTag(code,0,0,generic);
}

void KBloggerEditor::addHtmlLink ( const QString& target, const QString& name )
{
	insertTag (target,name,0,link);
}

void KBloggerEditor::insertTag (const QString& textA, const QString& textB, const QString& textC, tags tag) //HACK?
{
	QRegExp anytags;// tagToReplace;
	//QString replaceString;
	QString buff, buffA,buffB,buffC ;
	bool isCheckSpellingEnabled=VisualTextEditor->checkSpellingEnabled();

	VisualTextEditor->setCheckSpellingEnabled ( false );

//	kdDebug() << "KBloggerEditor::insertTag in the InitialText=\n" << VisualTextEditor->text()
//		  << "'\n the selection\n" << VisualTextEditor->selectedText() << "\n will be replaced with \n"<< text <<"\n"<<endl;
	
	if (tag){ // tag!=generic 
		anytags.setPattern("</?!?(--)?[\\w][^<>]*>");
		buffA=textA;
		buffA.remove(anytags);
		buffB=textB;
		buffB.remove(anytags);
		buffC=textC;
		buffC.remove(anytags);
		kdDebug() << "KBloggerEditor::insertTag\n textA=\n"<<textA<<"\nBuffA=\n"<<buffA<<"\ntextB=\n"<<textB<<"\nBuffB=\n"<<buffB<<"\ntextC=\n"<<textC << endl;
	}
	switch (tag)
	{
		case generic:
		{
			int pos=0;
			QRegExp testRegExp("(<[^<]*>)([^<]*)(</[^<]*>)");
			QValidator* validator = new QRegExpValidator(testRegExp,this);
			buff=textA;
			
			if ( validator->validate( buff,pos) != QValidator::Acceptable ) return;
			
		}break;
		case img:
		{
			buff="<img src='"+buffA+"'>";
			//tagToReplace.setPattern("<img src='\"([^<>]*)\"'>");
			//replaceString="&gt;img src='\"\\1\"'&lt;";
		}break;
		case imglinked:
		{
			buff="<a href=\""+buffA+"\"><img alt=\""+buffB+"\" src=\""+buffC+"\" ></a>";
			//tagToReplace.setPattern("<a href=\"[^<>]*\"><img src='\"[^<>]*\"></a>");
			//replaceString="&gta href=\"\\1*\"&lt;&gtimg src='\"\\2*\"&lt;&gt/a&lt;";
		}break;
		case link:
		{
			buff="<a href=\"" + buffA + "\">" + buffB + "</a>";
			//tagToReplace.setPattern("<a href=\"[^<>]*\">[^<>]*</a>");
			//replaceString=
		}break;
		case notags:
		{
			buff=buffA;
		}break;

		default: //No default!:P
			return;
	}
	VisualTextEditor->insert( buff );
	buff=VisualTextEditor->text();
/*
	
	kdDebug() << "KBloggerEditor::insertTag text='" << text << "' buff='"<<buff<<"'" <<endl;
	buff.replace ( QRegExp("&lt;"+startTag+"(.*)&gt;(.*)&lt;"+endTag+"&gt;"),"<"+starTag+"\\1>\\2<"+endTag+">");
	

*/
	buff.replace ( "&quot;","\"" );
	buff.replace ( "&lt;","<" );
	//buff.replace (tagToReplace,);
	buff.replace ( "&gt;",">" );

/*
	buff.replace (QRegExp("&lt;(code|b|i|u)(.*)&gt;(.*)&lt;/(a|span|code|b|i|u)&gt;"),
		      "<\\1\\2>\\3</\\4>");
	buff.replace (QRegExp("&lt;(a href=|span style=)&quot;(.*)&quot;&gt;(.*)&lt;/(a|span)&gt;")
		     ,"<\\1\"\\2\">\\3</\\4>");
*/	
	VisualTextEditor->setText ( buff );
	
	VisualTextEditor->setCheckSpellingEnabled ( isCheckSpellingEnabled );
}

QString KBloggerEditor::richtextToHtml(const QString& text)
{
	QString dirtyText=text;
	dirtyText.remove ( "<html><head><meta name=\"qrichtext\" content=\"1\" /></head>" );
	dirtyText.remove ( QRegExp ( "<body[^<]+>\n" ) );
	dirtyText.remove ( QRegExp ("</body></html>" ));

	QRegExp richtextToHtmlExp("(<span style=\")([^\".]*)(\">)([^<]*)(</span>)");
	
	while ( richtextToHtmlExp.search( dirtyText ) != -1 ) {
	  QString newSpanStart, newSpanEnd, SpanStartA, SpanStartB, SpanStartC, TextInSpan,SpanEnd;
          SpanStartA = richtextToHtmlExp.cap( 1 );
	  SpanStartB = richtextToHtmlExp.cap( 2 );
	  SpanStartC = richtextToHtmlExp.cap( 3 );
          TextInSpan = richtextToHtmlExp.cap( 4 );
          SpanEnd = richtextToHtmlExp.cap( 5 );
	  kdDebug() << "KBloggerEditor::richtextToHtml A='"<< SpanStartA <<"' B='" <<SpanStartB << "' C='" << SpanStartC << "' Text='" << TextInSpan<< "' SpanEnd='" << SpanEnd <<"'"<< endl;

	  if (SpanStartB.contains("font-size:16pt;font-weight:600",false)){ //H1
	    newSpanStart+="<h1>";
	    newSpanEnd="</h1>"+newSpanEnd;
	  }
	  if (SpanStartB.contains("font-size:12pt;font-weight:600",false)){ //H2
	    newSpanStart+="<h2>";
	    newSpanEnd="</h2>"+newSpanEnd;
	  }
	  if (SpanStartB.contains("font-size:9pt;font-weight:600",false)){ //H3
	    newSpanStart+="<h3>";
	    newSpanEnd="</h3>"+newSpanEnd;
	  }
	  if (SpanStartB.contains(QRegExp("[^;]?font-weight:600[^;]?"))){ //H4
	    newSpanStart+="<strong>";
	    newSpanEnd="</strong>"+newSpanEnd;
	  }
	  if (SpanStartB.contains("font-size:6pt;font-weight:600",false)){ //H5
	    newSpanStart+="<h5>";
	    newSpanEnd="</h5>"+newSpanEnd;
	  }
	  /*if (SpanStartB.contains("font-weight:",false)){//bold
	    newSpanStart+="<strong>";
	    newSpanEnd+="</strong>";
	  }*/
	  if (SpanStartB.contains("font-style:italic",false)){//italic
	    newSpanStart+="<em>";
	    newSpanEnd="</em>"+newSpanEnd;
	  }
	  if (SpanStartB.contains(QRegExp("text-decoration:[-\\sa-zA-Z]*line-through[-\\sa-zA-Z]*"))){
	    newSpanStart+="<del>";
	    newSpanEnd="</del>"+newSpanEnd;
	  }
	  if (SpanStartB.contains(QRegExp("text-decoration:[-\\sa-zA-Z]*underline[-\\sa-zA-Z]*"))){//underline
	    newSpanStart+="<u>";
	    newSpanEnd="</u>"+newSpanEnd;
	  }
	  if (SpanStartB.contains("font-family:Monospace",false)){//code
	    newSpanStart+="<code>";
	    newSpanEnd="</code>"+newSpanEnd;
	  }
	  if (SpanStartB.contains("font-size:6pt",false)){ //small
	    newSpanStart+="<small>";
	    newSpanEnd="</small>"+newSpanEnd;
	  }
	  if (SpanStartB.contains(QRegExp("font-size:9pt[^;]?"))){ //small
	    newSpanStart+="<big>";
	    newSpanEnd="</big>"+newSpanEnd;
	  }
	  if (SpanStartB.contains("vertical-align:sub",false)){ //sub
	    newSpanStart+="<sub>";
	    newSpanEnd="</sub>"+newSpanEnd;
	  }
	  if (SpanStartB.contains("vertical-align:super",false)){ //sub
	    newSpanStart+="<sup>";
	    newSpanEnd="</sup>"+newSpanEnd;
	  }

	  kdDebug() << "KBloggerEditor::richtextToHtml newSpanStart='" << newSpanStart << "' newSpanEnd='" << newSpanEnd <<"'"<<endl;
	  kdDebug() << "KBloggerEditor::richtextToHtml OldString=" << SpanStartA+SpanStartB+SpanStartC+TextInSpan+SpanEnd << "NewString" << newSpanStart+TextInSpan+newSpanEnd << endl;
	  dirtyText.replace(SpanStartA+SpanStartB+SpanStartC+TextInSpan+SpanEnd,newSpanStart+TextInSpan+newSpanEnd);
	  kdDebug() << "KBloggerEditor::richtextToHtml dirtyText='"<< dirtyText <<"'"<<endl;
    	}

	//
	dirtyText.replace(QRegExp("<p style=\"margin-left:40px;margin-right:40px\">(.*)?</p>"),"<blockquote>\\1</blockquote>"); //OK
	dirtyText.replace(QRegExp("<p style=\"margin-top:[\\d]*px\">"),"<p>"); //TEST
	return dirtyText;
}

QString KBloggerEditor::htmlToRichtext(const QString& text)
{
	QString dirtyText=text;
	dirtyText.replace(QRegExp("<del>(.*)</del>"),"<span style=\"text-decoration: line-through\">\\1</span>");
	return dirtyText;
}


void KBloggerEditor::loadPostSlot ( KBlog::BlogPosting& posting )
{
	kdDebug() << "KBloggerEditor::loadPostSlot()" << endl;
	TitleLineEdit->setText ( posting.title() );
	CategoryComboBox->setCurrentText ( posting.category() );
	VisualTextEditor->setText ( posting.content() );
	TimeStampKDateTime->setDateTime ( posting.creationDateTime() );
	TimeStampCheckBox->setChecked(true);
	disconnect ( dynamic_cast<QObject*> ( m_backend ), SIGNAL ( itemOnServer ( KBlog::BlogPosting& ) ),
	             dynamic_cast<QObject*> ( this ), SLOT ( loadPostSlot ( KBlog::BlogPosting& ) ) );
}

/*$SPECIALIZATION$*/

void KBloggerEditor::sendSlot()
{
	kdDebug() << "KBloggerEditor::sendSlot()" << endl;
	if ( !TitleLineEdit->isReadOnly() && TitleLineEdit->text().isEmpty() )
		if ( KMessageBox::questionYesNo ( this,
		                                  i18n ( "Are you sure you want to send without title?" ) ) !=3 ) // this is no good... the code for KDialog::ButtonCode Yes
			return;
	if ( VisualTextEditor->text().isEmpty() )
		if ( KMessageBox::questionYesNo ( this,
		                                  i18n ( "Are you sure you want to send without content?" ) ) !=3 ) // this is no good... the code for KDialog::ButtonCode Yes
			return;

	kdDebug() << "KBlog::BlogPosting()" << endl;
	SendButton->setEnabled ( false );
	m_statusBar->message(i18n("Sending post..."));
	connect ( m_backend, SIGNAL ( uploadPostId ( const int ) ),
	          this, SLOT ( handleSuccess ( const int ) ) );
	KBlog::BlogPosting *posting = new KBlog::BlogPosting();
	posting->setBlogID ( KBloggerConfig::blogId() );
	posting->setTitle ( TitleLineEdit->text() );

	syncEditors(0);
	
	QString richText = ContentTextHtmlEditor->text(); //Now Read the HTML Editor! (It's sanitized!)
	
	posting->setContent ( richText );
	posting->setPublish ( PublishCheckbox->isChecked() );
	posting->setCategory ( CategoryComboBox->currentText() );
	if ( TimeStampKDateTime->isEnabled() )
		posting->setCreationDateTime ( TimeStampKDateTime->dateTime() );
	KIO::TransferJob* uploadjob;
	if ( m_postid!=-1 )
	{
		kdDebug() << "Creating m_backend->createUploadJob()" << endl;
		posting->setPostID ( QString ( "%1" ).arg ( m_postid ) );
		uploadjob = m_backend->createUploadJob ( KURL::KURL ( m_postid ), posting );
	}
	else
	{
		kdDebug() << "Creating m_backend>createUploadNewJob()" << endl;
		uploadjob = m_backend->createUploadNewJob ( posting );
	}
	connect ( uploadjob, SIGNAL ( result ( KIO::Job* ) ),
	          m_backend, SLOT ( interpretUploadJob ( KIO::Job* ) ) );
	connect ( m_backend, SIGNAL ( uploadPostId ( const int ) ),
	          this, SLOT ( closeSignal ( const int ) ) );
}

void KBloggerEditor::closeSignal ( const int )
{
	emit editorClosed();
	close(true); //Destroy.
}

void KBloggerEditor::getCategoriesSlot ( const QString &name, const QString &description )
{
	kdDebug() << "KBloggerEditor::getCategoriesSlot()" << endl;
	
	if ( !CategoryComboBox->currentText().isEmpty() )
	{
		m_editCategory = CategoryComboBox->currentText();
		CategoryComboBox->insertItem ( name );
		CategoryComboBox->setEditText ( m_editCategory );
		nCategFetched++;
	}else{
		CategoryComboBox->insertItem ( name );
		nCategFetched=1;	
	}
	SendButton->setEnabled ( true );
	categDescription << description;
	m_statusBar->message(i18n("Retrieved %1 categories from your blog...").arg(nCategFetched));
}


void KBloggerEditor::syncEditors(QWidget* m_tab)
{
 static QWidget *prevPage=VisualPageWidget;
 if (!m_tab){ 
   if (VisualPageWidget-isShown())
     ContentTextHtmlEditor->setText( richtextToHtml(VisualTextEditor->text()) );
 }

 if (m_tab == VisualPageWidget && prevPage == HtmlPageWidget) // HTML Editor =>VISUAL editor 
   VisualTextEditor->setText(htmlToRichtext(ContentTextHtmlEditor->text()));

 if (m_tab == HtmlPageWidget && prevPage == VisualPageWidget) // Visual Editor => HTML editor.
   ContentTextHtmlEditor->setText(richtextToHtml(VisualTextEditor->text()));

 if (m_tab == PreviewPageWidget){ // HTML Editor =>Preview  & Visual Editor => Preview
   if (prevPage == VisualPageWidget)
     ContentTextHtmlEditor->setText(richtextToHtml(VisualTextEditor->text()));
   writeOnPreview( ContentTextHtmlEditor->text() );
 }
 prevPage=m_tab;

}

void KBloggerEditor::setCategDesc(int i)
{
	CategoryDescriptionKSqueezedTextLabel->setText(categDescription[i]);
}

void KBloggerEditor::writeOnPreview(const QString& text)
{
	PreviewBrowser->begin();
	PreviewBrowser->openStream( "text/html", KURL() );
	if (cssUrl->isValid())
		PreviewBrowser->setUserStyleSheet(*cssUrl);
	PreviewBrowser->writeStream( QCString( "<html><body><p>"+text+"</p></body></html>" ) );
	PreviewBrowser->closeStream();
}

bool KBloggerEditor::close(bool destruct)
{
  emit editorClosed();
  kdDebug()<< "KBloggerEditor::close" << endl;
  QWidget::close(true);
}





#include "kbloggereditor.moc"
