/***************************************************************************
 *   Copyright (C) 2005 by Christian Weilbach   *
 *   christian_weilbach@web.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kbloggermenu.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qsize.h>
#include <qpixmap.h>
#include <qcursor.h>
#include <qprogressbar.h>
#include <qstring.h>
#include <qpoint.h>

#include <kglobal.h>
#include <klocale.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kmessagebox.h>

#include "kbloggerconfig.h"

const int MAXPOSTS=10;

KBloggerMenu::KBloggerMenu ( KBlog::APIBlog* backend, KBlogger* parent, const char* name ) :
		KPopupMenu ( parent, name ), m_backend ( backend ), m_parent ( parent ), m_pos ( QPoint() ), m_fullMenu ( false )
{
	kdDebug() << "KBloggerMenu::KBloggerMenu()" << endl;

	connect ( m_backend, SIGNAL ( itemOnServer ( KBlog::BlogPosting& ) ),
	          this, SLOT ( addPostingSlot ( KBlog::BlogPosting& ) ) );
	connect ( m_backend, SIGNAL ( fetchingPostsFinished() ),
	          this, SLOT ( createMenuSlot() ) );
}

void KBloggerMenu::populateMenu()
{
	kdDebug() << "KBloggerMenu::populateMenu()" << endl;

	//Remove all Item in the menu
	clear();
	postFetcherBar=0;
	getPostLabel=0;

	//Insert Label & ProgressBar for fetching blog's entry.
	if ( m_fullMenu )
	{
		getPostLabel=new QLabel ( i18n ( "Fetching posts..." ),this );
		insertItem ( getPostLabel );
		postFetcherBar=new QProgressBar ( MAXPOSTS,this,"Post fetcher QProgressBar" );
		insertItem ( postFetcherBar );
		postFetcherBar->setPercentageVisible ( true );
		postFetcherBar->setCenterIndicator ( true );
		postFetcherBar->setProgress ( 0 );
		insertSeparator();
		postings.clear(); 
		//postings = QValueList<KBlog::BlogPosting>(); //Reinitialize
		KIO::TransferJob* job = m_backend->createListItemsJob ( KURL::KURL ( KBloggerConfig::blogId() ) );
		connect ( job, SIGNAL ( result ( KIO::Job* ) ),
		          m_backend, SLOT ( interpretListItemsJob ( KIO::Job* ) ) );
		connect ( m_backend, SIGNAL ( error( const QString&)),
		          this, SLOT ( stopFetchingProgressBar()));
	}

	//Insert Other Entry.
	const QPixmap bugIcon  = SmallIcon ( QString::fromLatin1 ( "bug" ) );;
	const QPixmap logoIcon = SmallIcon ( QString::fromLatin1 ( "kblogger" ) );
	const QPixmap helpIcon = SmallIcon ( QString::fromLatin1 ( "help" ) );
	const QPixmap confIcon = SmallIcon ( QString::fromLatin1 ( "configure" ) );
	const QPixmap fileIcon = SmallIcon ( QString::fromLatin1 ( "fileopen" ) );

	insertTitle ( logoIcon, i18n ( "KBlogger" ), 0, 0 );
	if ( m_backend->interfaceName() == "MetaWeblog API" )//API Blogger can't upload files! 
		insertItem ( fileIcon, i18n ( "File upload" ), m_parent, SLOT ( uploadSlot() ) );

	insertItem ( confIcon, i18n ( "Configuration" ), this, SLOT ( configurateSlot() ) );
	insertItem ( bugIcon, i18n ( "Bugs or Wishes" ), this, SLOT ( bugSlot() ) );
	insertItem ( helpIcon, i18n ( "Help" ), this, SLOT ( helpSlot() ) );
	insertItem ( logoIcon, i18n ( "About KBlogger" ), this, SLOT ( aboutSlot() ) );
	
	//Show menu.
	popup ( m_pos ); //Using popup instead exec => i can add other widget!!!
}

void KBloggerMenu::addPostingSlot ( KBlog::BlogPosting& posting )
{
	kdDebug() << "KBloggerMenu::addPostingSlot()" << endl;
	postings.append ( posting );
}

void KBloggerMenu::createMenuSlot()
{
	if ( ! m_fullMenu || postFetcherBar==0)
	  return;

	if (isHidden()) return;

	const QPixmap entryIcon = SmallIcon ( QString::fromLatin1 ( "package_editors" ));

	QValueList<KBlog::BlogPosting>::ConstIterator it = postings.begin();
	int i=1;
	insertTitle ( entryIcon, i18n ( "Fetched entries:" )); //Add a title
	for ( ; it != postings.end(); ++it )
	{
		// Use a submenu, if more options are available
		KPopupMenu *submenu = new KPopupMenu(this);
		insertItem( ( *it ).title(), submenu );
		submenu->insertItem( i18n( "Edit post" ), m_parent, SLOT(editorSlot(int)), 0, ( *it ).postID().toInt() );
		submenu->insertItem( i18n( "Delete post" ), this, SLOT(deleteSlot(int)), 0, ( *it ).postID().toInt()+1 );
		if ( i++==MAXPOSTS ) break;
	}
	
	removeItemAt(2); //remove & delete QProgressBar
	removeItemAt(1); //remove & delete QLabel "Fetching posts"
	postFetcherBar=0;
	getPostLabel=0;
	hide();
	//adjustSize(); //adjust popup menu size!
	popup(m_pos);
}

void KBloggerMenu::bugSlot()
{
	m_parent->reportBug();
}

void KBloggerMenu::configurateSlot()
{
	m_parent->preferences();
}

void KBloggerMenu::helpSlot()
{
	m_parent->help();
}

void KBloggerMenu::aboutSlot()
{
	m_parent->about();
}

void KBloggerMenu::stopFetchingProgressBar()
{
	if (!postFetcherBar) return;
	getPostLabel->setText("<center><b><font color=red>ERROR: KBlogger is not able to fetch your post.</font></b></center>");
	removeItemAt(2); //remove QProgressBar.
	postFetcherBar=0;
	adjustSize(); //adjust popup menu size!
}

void KBloggerMenu::mousePressEvent ( QMouseEvent *e )
{
	if ( e->button() == QMouseEvent::RightButton) return;
}

void KBloggerMenu::deleteSlot ( int postid )
{
	int m_postid=postid-1;
	kdDebug() << "KBloggerMenu::deleteSlot()" << endl;
	int i=KMessageBox::questionYesNo ( this, i18n ( "You are about to delete the post %1. 'OK' to delete, 'Cancel' to stop." ).arg ( m_postid ),i18n("Delete post..."),i18n("OK"),i18n("Cancel") );
	if (i==4 ) //CANCEL
		return;
	m_backend->createRemoveJob ( KURL::KURL ( KBloggerConfig::blogId() ), QString ( "%1" ).arg ( m_postid ) );
}
