/***************************************************************************
 *   Copyright (C) 2005 by Christian Weilbach   *
 *   christian_weilbach@web.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#define MAX_FILE_SIZE_WARNING 2048000 

#include "kbloggerupload.h"

#include <qfileinfo.h>
#include <qlabel.h>
#include <qlayout.h>

#include <kfiledialog.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>
#include <kurlrequester.h>

#include "kbloggerconfigdialog.h"
#include "kbloggerconfig.h"
#include "kbloggerimagemanager.h"

KBloggerUpload::KBloggerUpload( KBlog::APIBlog* backend, QString& filename,  mimeType filter,QDir* tmpDir,QWidget *parent, const char *name)
    :KBloggerUploadBlank(parent, name), m_backend(backend), m_media (new KBlog::BlogMedia()), m_imageManager(0), m_tmpDir(tmpDir)
{
	kdDebug() << "KBloggerUpload::KBloggerUpload()" << endl;
	
	
	if( !filename.isEmpty() )
		FilekURLRequester->setURL( filename );
	
	switch (filter){
		case Images:
		{
			genericGroupBox->setHidden(false);
			FilekURLRequester->setFilter( "Images ( *.png *.jpg *.gif )" );
			m_imageManager= new KBloggerImageManager(m_tmpDir,genericGroupBox,"Image Manager");
			m_imageManager->show();
			genericGroupBoxLayout->addWidget( m_imageManager, 0, 0 );
			connect(m_imageManager,SIGNAL(imageResized(const QString&)),SLOT(setFileNameOnServer(const QString&)));
			adjustSize();
		}	
		break;
	
		default:
			genericGroupBox->setHidden(true);
			adjustSize();
	}
	show();
}

KBloggerUpload::~KBloggerUpload(){
	kdDebug() << "KBloggerUpload::~KBloggerUpload()" << endl;
	delete m_media;
}

void KBloggerUpload::uploadSlot(){

	kdDebug() << "KBloggerUpload::uploadSlot()" << endl;

	if( FilekURLRequester->url().isEmpty() || FilekURLRequester->url().isEmpty() ) {
		KMessageBox::error( 0, i18n( "You have to enter a name and a valid URL." )  );
		return;
	}

	KIO::TransferJob *fileJob = KIO::get( FilekURLRequester->url() );

	connect( fileJob, SIGNAL( result( KIO::Job* ) ),
			this, SLOT( handleFileJobResult( KIO::Job* ) ) );
	connect( fileJob, SIGNAL( data( KIO::Job*,  const QByteArray & ) ),
			this, SLOT( handleFileJobData( KIO::Job*,  const QByteArray & ) ) );
	connect( fileJob, SIGNAL( mimetype( KIO::Job*,  const QString & ) ),
			this, SLOT( handleFileJobMimetype( KIO::Job*,  const QString & ) ) );

	this->close();
}

void KBloggerUpload::handleFileJobData( KIO::Job* job, const QByteArray& data ){
	kdDebug() << "KBloggerUpload::handleFileJobData()"<< data << endl;
	int m_size= ( m_media->data() ).size();
	int size= data.size();
	( m_media->data() ).resize( m_size + size );
	for( int i=0; i< size; i++ ) // FIXME ?
		( m_media->data() )[ m_size + i ] = data[ i ];
}

void KBloggerUpload::handleFileJobMimetype( KIO::Job* job, const QString& mimetype ){
	kdDebug() << "KBloggerUpload::handleFileJobMimetype() mimetype=" << mimetype <<endl;
	m_media->setMimetype( mimetype );
}

void KBloggerUpload::handleFileJobResult( KIO::Job* job ){
	if( job->error() ){
	        job->showErrorDialog();
		return;
	}
	kdDebug() << "KBloggerUpload::handleFileJobResult() got file successfully" << endl;
	m_media->setBlogID( KBloggerConfig::blogId() );
	m_media->setName( NameLineEdit->text() );
	KIO::Job* uploadJob=m_backend->createUploadMediaJob( KURL::KURL( KBloggerConfig::url() ), m_media );
	connect( uploadJob, SIGNAL( result( KIO::Job* ) ),
			m_backend, SLOT( interpretUploadMediaJob( KIO::Job* ) ) );
	connect (m_backend, SIGNAL(mediaInfoRetrieved( const QString& )), //Url
		parent(), SLOT(fileUploadedSlot( const QString& ))); //url
}

void KBloggerUpload::setFileNameOnServer(const QString& filename)
{
	int filesize;
	if (filename.isEmpty()) return; 
	FilekURLRequester->setURL(filename);
	QFileInfo fileToUpload(filename);
	NameLineEdit->setText(fileToUpload.fileName());
	filesize=fileToUpload.size();
	if ( filesize > MAX_FILE_SIZE_WARNING ){
		filesizeLabel->setText(i18n ("File size: <font color='red'>" + QString::number(filesize) + "</font> bytes"));
		KMessageBox::sorry ( this, i18n ( "Please don't upload big files!!" ) );
	}else{
		filesizeLabel->setText(i18n ("File size: <font color='green'>" + QString::number(filesize) + "</font> bytes"));
	}
	if (m_imageManager)
		m_imageManager->setImage(filename);
}



#include "kbloggerupload.moc"
